@@ -3,14 +3,17 @@ Module to handle agent callbacks
 """
 
 import inspect
+import logging
 from typing import Any, Dict, Optional, List, Callable
 from functools import wraps
+import traceback
 
 from llama_index.core.callbacks.base_handler import BaseCallbackHandler
 from llama_index.core.callbacks.schema import CBEventType, EventPayload
 
 from .types import AgentStatusType
 
+
 def wrap_callback_fn(callback):
     """
     Wrap a callback function to ensure it only receives the parameters it can accept.
@@ -34,6 +37,47 @@ def wrap_callback_fn(callback):
 
     return new_callback
 
+
+def _extract_content_from_response(response) -> str:
+    """
+    Extract text content from various LLM response formats.
+
+    Handles different provider response objects and extracts the text content consistently.
+
+    Args:
+        response: Response object from LLM provider
+
+    Returns:
+        str: Extracted text content
+    """
+    # Handle case where response is a string
+    if isinstance(response, str):
+        return response
+
+    # Handle ChatMessage objects with blocks (Anthropic, etc.)
+    if hasattr(response, "blocks") and response.blocks:
+        text_parts = []
+        for block in response.blocks:
+            if hasattr(block, "text"):
+                text_parts.append(block.text)
+        return "".join(text_parts)
+
+    # Handle responses with content attribute
+    if hasattr(response, "content"):
+        return str(response.content)
+
+    # Handle responses with message attribute that has content
+    if hasattr(response, "message") and hasattr(response.message, "content"):
+        return str(response.message.content)
+
+    # Handle delta attribute for streaming responses
+    if hasattr(response, "delta"):
+        return str(response.delta)
+
+    # Fallback to string conversion
+    return str(response)
+
+
 class AgentCallbackHandler(BaseCallbackHandler):
     """
     Callback handler to track agent status
@@ -147,30 +191,36 @@ class AgentCallbackHandler(BaseCallbackHandler):
     def _handle_event(
         self, event_type: CBEventType, payload: Dict[str, Any], event_id: str
     ) -> None:
-        if event_type == CBEventType.LLM:
-            self._handle_llm(payload, event_id)
-        elif event_type == CBEventType.FUNCTION_CALL:
-            self._handle_function_call(payload, event_id)
-        elif event_type == CBEventType.AGENT_STEP:
-            self._handle_agent_step(payload, event_id)
-        elif event_type == CBEventType.EXCEPTION:
-            print(f"Exception event in handle_event: {payload.get(EventPayload.EXCEPTION)}")
-        else:
-            print(f"Unknown event type: {event_type}, payload={payload}")
+        try:
+            if event_type == CBEventType.LLM:
+                self._handle_llm(payload, event_id)
+            elif event_type == CBEventType.FUNCTION_CALL:
+                self._handle_function_call(payload, event_id)
+            elif event_type == CBEventType.AGENT_STEP:
+                self._handle_agent_step(payload, event_id)
+            else:
+                pass
+        except Exception as e:
+            logging.error(f"Exception in callback handler: {e}")
+            logging.error(f"Traceback: {traceback.format_exc()}")
+            # Continue execution to prevent callback failures from breaking the agent
 
     async def _ahandle_event(
         self, event_type: CBEventType, payload: Dict[str, Any], event_id: str
     ) -> None:
-        if event_type == CBEventType.LLM:
-            await self._ahandle_llm(payload, event_id)
-        elif event_type == CBEventType.FUNCTION_CALL:
-            await self._ahandle_function_call(payload, event_id)
-        elif event_type == CBEventType.AGENT_STEP:
-            await self._ahandle_agent_step(payload, event_id)
-        elif event_type == CBEventType.EXCEPTION:
-            print(f"Exception event in ahandle_event: {payload.get(EventPayload.EXCEPTION)}")
-        else:
-            print(f"Unknown event type: {event_type}, payload={payload}")
+        try:
+            if event_type == CBEventType.LLM:
+                await self._ahandle_llm(payload, event_id)
+            elif event_type == CBEventType.FUNCTION_CALL:
+                await self._ahandle_function_call(payload, event_id)
+            elif event_type == CBEventType.AGENT_STEP:
+                await self._ahandle_agent_step(payload, event_id)
+            else:
+                pass
+        except Exception as e:
+            logging.error(f"Exception in async callback handler: {e}")
+            logging.error(f"Traceback: {traceback.format_exc()}")
+            # Continue execution to prevent callback failures from breaking the agent
 
     # Synchronous handlers
     def _handle_llm(
@@ -179,55 +229,63 @@ class AgentCallbackHandler(BaseCallbackHandler):
         event_id: str,
     ) -> None:
         if EventPayload.MESSAGES in payload:
-            response = str(payload.get(EventPayload.RESPONSE))
-            if response and response not in ["None", "assistant: None"]:
+            response = payload.get(EventPayload.RESPONSE)
+            if response and str(response) not in ["None", "assistant: None"]:
                 if self.fn:
+                    # Convert response to consistent dict format
+                    content = _extract_content_from_response(response)
                     self.fn(
                         status_type=AgentStatusType.AGENT_UPDATE,
-                        msg=response,
+                        msg={"content": content},
                         event_id=event_id,
                     )
         elif EventPayload.PROMPT in payload:
-            prompt = str(payload.get(EventPayload.PROMPT))
+            prompt = payload.get(EventPayload.PROMPT)
             if self.fn:
+                # Convert prompt to consistent dict format
+                content = str(prompt) if prompt else ""
                 self.fn(
                     status_type=AgentStatusType.AGENT_UPDATE,
-                    msg=prompt,
+                    msg={"content": content},
                     event_id=event_id,
                 )
         else:
-            print(
-                f"vectara-agentic llm callback: no messages or prompt found in payload {payload}"
-            )
+            pass
 
     def _handle_function_call(self, payload: dict, event_id: str) -> None:
-        if EventPayload.FUNCTION_CALL in payload:
-            fcall = str(payload.get(EventPayload.FUNCTION_CALL))
-            tool = payload.get(EventPayload.TOOL)
-            if tool:
-                tool_name = tool.name
-                if self.fn:
+        try:
+            if EventPayload.FUNCTION_CALL in payload:
+                fcall = payload.get(EventPayload.FUNCTION_CALL)
+                tool = payload.get(EventPayload.TOOL)
+
+                if tool:
+                    tool_name = tool.name
+                    if self.fn:
+                        self.fn(
+                            status_type=AgentStatusType.TOOL_CALL,
+                            msg={"tool_name": tool_name, "arguments": fcall},
+                            event_id=event_id,
+                        )
+
+            elif EventPayload.FUNCTION_OUTPUT in payload:
+                response = payload.get(EventPayload.FUNCTION_OUTPUT)
+                tool = payload.get(EventPayload.TOOL)
+
+                if tool and self.fn:
                     self.fn(
-                        status_type=AgentStatusType.TOOL_CALL,
-                        msg=f"Executing '{tool_name}' with arguments: {fcall}",
+                        status_type=AgentStatusType.TOOL_OUTPUT,
+                        msg={"tool_name": tool.name, "content": response},
                         event_id=event_id,
                     )
-        elif EventPayload.FUNCTION_OUTPUT in payload:
-            response = str(payload.get(EventPayload.FUNCTION_OUTPUT))
-            if self.fn:
-                self.fn(
-                    status_type=AgentStatusType.TOOL_OUTPUT,
-                    msg=response,
-                    event_id=event_id,
-                )
-        else:
-            print(
-                f"Vectara-agentic callback handler: no function call or output found in payload {payload}"
-            )
+
+        except Exception as e:
+            logging.error(f"Exception in _handle_function_call: {e}")
+            logging.error(f"Traceback: {traceback.format_exc()}")
+            # Continue execution to prevent callback failures from breaking the agent
 
     def _handle_agent_step(self, payload: dict, event_id: str) -> None:
         if EventPayload.MESSAGES in payload:
-            msg = str(payload.get(EventPayload.MESSAGES))
+            msg = payload.get(EventPayload.MESSAGES)
             if self.fn:
                 self.fn(
                     status_type=AgentStatusType.AGENT_STEP,
@@ -235,90 +293,98 @@ class AgentCallbackHandler(BaseCallbackHandler):
                     event_id=event_id,
                 )
         elif EventPayload.RESPONSE in payload:
-            response = str(payload.get(EventPayload.RESPONSE))
+            response = payload.get(EventPayload.RESPONSE)
             if self.fn:
                 self.fn(
                     status_type=AgentStatusType.AGENT_STEP,
                     msg=response,
                     event_id=event_id,
                 )
-        else:
-            print(
-                f"Vectara-agentic agent_step: no messages or prompt found in payload {payload}"
-            )
 
     # Asynchronous handlers
     async def _ahandle_llm(self, payload: dict, event_id: str) -> None:
         if EventPayload.MESSAGES in payload:
-            response = str(payload.get(EventPayload.RESPONSE))
-            if response and response not in ["None", "assistant: None"]:
+            response = payload.get(EventPayload.RESPONSE)
+            if response and str(response) not in ["None", "assistant: None"]:
                 if self.fn:
+                    # Convert response to consistent dict format
+                    content = _extract_content_from_response(response)
                     if inspect.iscoroutinefunction(self.fn):
                         await self.fn(
                             status_type=AgentStatusType.AGENT_UPDATE,
-                            msg=response,
+                            msg={"content": content},
                             event_id=event_id,
                         )
                     else:
                         self.fn(
                             status_type=AgentStatusType.AGENT_UPDATE,
-                            msg=response,
+                            msg={"content": content},
                             event_id=event_id,
                         )
         elif EventPayload.PROMPT in payload:
-            prompt = str(payload.get(EventPayload.PROMPT))
+            prompt = payload.get(EventPayload.PROMPT)
             if self.fn:
+                # Convert prompt to consistent dict format
+                content = str(prompt) if prompt else ""
                 self.fn(
                     status_type=AgentStatusType.AGENT_UPDATE,
-                    msg=prompt,
+                    msg={"content": content},
                     event_id=event_id,
                 )
-        else:
-            print(
-                f"vectara-agentic llm callback: no messages or prompt found in payload {payload}"
-            )
 
     async def _ahandle_function_call(self, payload: dict, event_id: str) -> None:
-        if EventPayload.FUNCTION_CALL in payload:
-            fcall = str(payload.get(EventPayload.FUNCTION_CALL))
-            tool = payload.get(EventPayload.TOOL)
-            if tool:
-                tool_name = tool.name
-                if self.fn:
+        try:
+            if EventPayload.FUNCTION_CALL in payload:
+                fcall = payload.get(EventPayload.FUNCTION_CALL)
+                tool = payload.get(EventPayload.TOOL)
+
+                if tool and self.fn:
                     if inspect.iscoroutinefunction(self.fn):
                         await self.fn(
                             status_type=AgentStatusType.TOOL_CALL,
-                            msg=f"Executing '{tool_name}' with arguments: {fcall}",
+                            msg={"tool_name": tool.name, "arguments": fcall},
                             event_id=event_id,
                         )
                     else:
                         self.fn(
                             status_type=AgentStatusType.TOOL_CALL,
-                            msg=f"Executing '{tool_name}' with arguments: {fcall}",
+                            msg={"tool_name": tool.name, "arguments": fcall},
                             event_id=event_id,
                         )
-        elif EventPayload.FUNCTION_OUTPUT in payload:
-            if self.fn:
-                response = str(payload.get(EventPayload.FUNCTION_OUTPUT))
-                if inspect.iscoroutinefunction(self.fn):
-                    await self.fn(
-                        status_type=AgentStatusType.TOOL_OUTPUT,
-                        msg=response,
-                        event_id=event_id,
-                    )
-                else:
-                    self.fn(
-                        status_type=AgentStatusType.TOOL_OUTPUT,
-                        msg=response,
-                        event_id=event_id,
-                    )
-        else:
-            print(f"No function call or output found in payload {payload}")
+
+            elif EventPayload.FUNCTION_OUTPUT in payload:
+                response = payload.get(EventPayload.FUNCTION_OUTPUT)
+                tool = payload.get(EventPayload.TOOL)
+
+                if tool and self.fn:
+                    if inspect.iscoroutinefunction(self.fn):
+                        await self.fn(
+                            status_type=AgentStatusType.TOOL_OUTPUT,
+                            msg={
+                                "tool_name": tool.name,
+                                "content": response,
+                            },
+                            event_id=event_id,
+                        )
+                    else:
+                        self.fn(
+                            status_type=AgentStatusType.TOOL_OUTPUT,
+                            msg={
+                                "tool_name": tool.name,
+                                "content": response,
+                            },
+                            event_id=event_id,
+                        )
+
+        except Exception as e:
+            logging.error(f"Exception in _ahandle_function_call: {e}")
+            logging.error(f"Traceback: {traceback.format_exc()}")
+            # Continue execution to prevent callback failures from breaking the agent
 
     async def _ahandle_agent_step(self, payload: dict, event_id: str) -> None:
         if EventPayload.MESSAGES in payload:
             if self.fn:
-                msg = str(payload.get(EventPayload.MESSAGES))
+                msg = payload.get(EventPayload.MESSAGES)
                 if inspect.iscoroutinefunction(self.fn):
                     await self.fn(
                         status_type=AgentStatusType.AGENT_STEP,
@@ -333,7 +399,7 @@ class AgentCallbackHandler(BaseCallbackHandler):
                     )
         elif EventPayload.RESPONSE in payload:
             if self.fn:
-                response = str(payload.get(EventPayload.RESPONSE))
+                response = payload.get(EventPayload.RESPONSE)
                 if inspect.iscoroutinefunction(self.fn):
                     await self.fn(
                         status_type=AgentStatusType.AGENT_STEP,
@@ -346,5 +412,3 @@ class AgentCallbackHandler(BaseCallbackHandler):
                         msg=response,
                         event_id=event_id,
                     )
-        else:
-            print(f"No messages or prompt found in payload {payload}")
